﻿using apifon_sdk_csharp.com.apifon.sdk.resource.helpers;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    public class OAuthConfigurationBuilder
    {
        private readonly string grantType;
        private readonly string clientId;
        private readonly HashSet<string> scope;
        private string clientSecret;
        private S256PKCEChallenge pkceChallenge;
        private string redirectUri;
        private string state;

        private OAuthConfigurationBuilder(string clientId, OAuthGrantType grantType)
        {
            this.clientId = clientId;
            this.grantType = grantType.getGrantTypeAsString();
            this.scope = new HashSet<string>();
        }

        public static OAuthConfigurationBuilder getClientCredentialsBuilder(string clientId, string clientSecret)
        {
            OAuthConfigurationBuilder oauthRequestBuilder = new OAuthConfigurationBuilder(clientId, OAuthGrantType.CLIENT_CREDENTIALS);
            oauthRequestBuilder.clientSecret = clientSecret;
            return oauthRequestBuilder;
        }

        public OAuthConfigurationBuilder withScope(OpenIDScope scope)
        {
            this.scope.Add(scope.getScopeValue());
            return this;
        }

        public OAuthConfigurationBuilder withScopes(Collection<string> scopes)
        {
            this.scope.UnionWith(scopes);
            return this;
        }

        public OAuthConfigurationBuilder withRedirectUri(string redirectUri)
        {
            this.redirectUri = redirectUri;
            return this;
        }

        public OAuthConfigurationBuilder withRedirectUri(Uri redirectUri, bool forceUrlEncoding)
        {
            this.redirectUri = redirectUri.ToString();
            return this;
        }

        public OAuthConfigurationBuilder withState(string state)
        {
            if (string.IsNullOrEmpty(state)) { throw new System.ArgumentNullException("state parameter can not be null"); }
            this.state = state;
            return this;
        }

        public OAuthConfiguration build()
        {
            return new OAuthConfiguration(this);
        }

        public class OAuthConfiguration 
        {
            private readonly string grantType;
            private readonly string clientId;
            private readonly HashSet<string> scope;
            private readonly string clientSecret;
            private readonly S256PKCEChallenge pkceChallenge;
            private readonly string redirectUri;
            private readonly string state;

            public OAuthConfiguration(OAuthConfigurationBuilder builder)
            {
                this.grantType = builder.grantType;
                this.clientId = builder.clientId;
                this.scope = new HashSet<string>(builder.scope);
                this.clientSecret = builder.clientSecret;
                if (builder.pkceChallenge != null)
                {
                    this.pkceChallenge = new S256PKCEChallenge(builder.pkceChallenge.getCodeVerifier());
                }
                else
                {
                    this.pkceChallenge = null;
                }
                this.redirectUri = builder.redirectUri;
                if (builder.state == null)
                {
                    var randomBytes = System.Text.Encoding.UTF8.GetBytes(Guid.NewGuid().ToString());
                    this.state = System.Convert.ToBase64String(randomBytes);
                }
                else
                {
                    this.state = builder.state;
                }
            }

            public string getGrantType()
            {
                return grantType;
            }

            public string getClientId()
            {
                return clientId;
            }

            public HashSet<string> getScope()
            {
                return this.scope;
            }

            public string getScopesAsString()
            {
                return string.Join("+", this.scope);
            }

            public string getClientSecret()
            {
                return clientSecret;
            }

            public S256PKCEChallenge getPkceChallenge()
            {
                return pkceChallenge;
            }

            public string getRedirectUri()
            {
                return redirectUri;
            }

            public string getState()
            {
                return state;
            }
        }
    }
}
